#===============================================================================
# Calculate Excess Path
# @author: Gaurav Bharaj
# @summary: Calculate bonus based on two criteria:
#            1) Excess Path : Difference between actually Manhattan distance and 
#            2) Pointing Error
#===============================================================================
import math as math
import string
import sys
import os.path

class CreateErrorLogs:
    ''' Creates Error Logs '''
   
    def __init__(self, dataDirPath, subjectName):
        ''' Initiate the file path variables & Calculates the various file paths '''
        
        self.dataDirPath = dataDirPath
        if os.path.isdir(self.dataDirPath) == False:
            print 'Invalid data directory path'
            exit(0)
            
        self.subjectName = subjectName
        
        self.logFilePath = self.dataDirPath + subjectName + '/parsed/'
        if os.path.isdir(self.logFilePath) == False:
            print 'Invalid Parsed directory path'
            exit(0)
            
        self.logNamesFilePath = self.logFilePath +  subjectName +'_LogNames.log'
        
        if os.path.isfile(self.logNamesFilePath) == False:
            print 'Log-Names files not found on the path: ',
            print self.logNamesFilePath
            exit(0)
            
        self.excessPathFilePath = self.logFilePath + subjectName + '_ExcessPath.log'
        self.pointErrorFilePath = self.logFilePath + subjectName + '_PointError.log'
        self.meanExcessPath = self.logFilePath + subjectName + '_MeanExcessPath.log'
        self.meanPointingError = self.logFilePath + subjectName + '_MeanPointingError.log'
        
        self.delvPerEnv = 15
        self.totalEnv = 8
    
    def __normalize(self, angle):
        ''' Bring the angle in range(0, 360) '''
        
        while angle < 0.0:
            angle += 360.0
        
        while angle >= 360.0:
            angle -= 360.0
            
        return angle
    
    def __clamp(self, angle):
        ''' Clamp the angle in range(0,180) '''
        
        if angle > 360.0:
            print 'Invalid angle input while clamping'
            exit(1)
            
        if angle > 180.0:
            angle = 360.0 - angle
            
        return angle

    def __calculatePointError(self, ot, pt):
        ''' Calculates the pointing error '''

        pt = self.__normalize(pt)
        ot = self.__normalize(ot)                
        return self.__clamp(math.fabs(ot - pt))
    
    def __calculateMeanDirection(self, thetas, n):
        ''' Calculates the mean direction. Reference: Fisher Page 31'''
        C = 0.0
        S = 0.0

        for theta in thetas:
            C += math.cos(math.radians(theta))
            S += math.sin(math.radians(theta))
        
        if S>0.0 and C>0.0:
            thetaBar = math.atan(S/C)
        elif C<0.0:
            thetaBar = math.atan(S/C) + math.pi
        elif S<0.0 and C>0.0:
            thetaBar = math.atan(S/C) + 2*math.pi

        return thetaBar
    
    def parseAndCreate(self):
        ''' Calculate the pointing error and the excess path '''
        logNamesFile_itr = open(self.logNamesFilePath, "r")
        excessPathFile = open(self.excessPathFilePath, 'w')
        pointErrorFile = open(self.pointErrorFilePath, 'w')
        deliverNum = 0
        totalThetaError = 0.0
        while (1):
            try:
                line = logNamesFile_itr.next().strip()
                logFileName = self.logFilePath + line
                deliverNum += 1
                
                logFile_itr = open(logFileName, 'r')
                previous_x = 0.0
                previous_y = 0.0
                current_x = 0.0
                current_y = 0.0
                
                previousInitiated = False
                totalEuclidianDist =  0.0
                manhattanDist = 0.0
                optimalTheta = 0.0
                playerTheta = 0.0
                goalInit = False
                
                while(1):
                    try:
                        words = string.split(logFile_itr.next().strip())
                        
                        # get for Manhattan distance calculation
                        if words[0] == 'GOAL':
                            goal_x = float(words[2])
                            goal_y = float(words[3])
                            optimalTheta = float(words[4])
                            goalInit = True
                        
                        # Calculate the point error'
                        if words[0] == 'COMPASS':
                            playerTheta = float(words[2])
                            error = self.__calculatePointError(optimalTheta, playerTheta)
                            pointErrorFile.write(str(deliverNum) + '    ' + str(error) + '\n')
                            totalThetaError += error
                            
                        # Calculate total euclidian distance
                        if words[0] == 'MOVE':
                            if previousInitiated == False:
                                previous_x = float(words[2])
                                previous_y = float(words[3])
                                
                                if goalInit == True:
                                    manhattanDist = math.fabs(goal_x - previous_x) + math.fabs(goal_y - previous_y)
                                else:
                                    print 'Error in reading file'
                                
                                previousInitiated = True
                                continue
                            
                            current_x = float(words[2])
                            current_y = float(words[3])
                            
                            totalEuclidianDist += math.sqrt((current_x - previous_x)**2 + (current_y - previous_y)**2)
                            previous_x = current_x
                            previous_y = current_y
                        
                    except StopIteration:
                        print 'End of ' +logFileName+' reached'
#                        print 'Manhattan distance: %f' %(manhattanDist)
#                        print 'Euclidian distance: %f' %(totalEuclidianDist)
#                        print 'Error: %f' %(totalEuclidianDist - manhattanDist)
                        
                        # Log results to file
                        #                       DeliveryNumber                             Error
                        excessPathFile.write(str(deliverNum) + '    ' + str(totalEuclidianDist - manhattanDist) + '\n')
                        break
                    
                logFile_itr.close()
            except StopIteration:
                print 'All Logs process..'
                break
        logNamesFile_itr.close()
        excessPathFile.close()
        
        pointErrorFile.write('Total Error: ' + str(totalThetaError))
        pointErrorFile.close()
        print 'Excess path calculation complete'
    
    def calculateMeanExcessPath(self):
        ''' Calculates the average excess paths '''
    
        dist = []
        readItr = open(self.excessPathFilePath, 'r')
        
        while(1):
            try:
                words = string.split(readItr.next().strip())
                dist.append(words[1])
            except StopIteration:
                print 'Excess path file parsed'
                break
        
        readItr.close()
        if len(dist) != self.delvPerEnv*self.totalEnv:
            print 'Some deliveries are missing'
            exit(1)
        
        # Write to a file
        writeItr = open(self.meanExcessPath, 'w')
        totalDist = 0.0
        for d in range (0, self.delvPerEnv):
            for idx in range(0+d, self.delvPerEnv*self.totalEnv):
                if (idx-d)%15 == 0:
                    totalDist += float(dist[idx])
            writeItr.write(str(d) + '    ' + str(totalDist/self.totalEnv) + '\n')
            totalDist = 0.0
                
        writeItr.close()      
            
    def calculateMeanPointingError(self):
        ''' Calculates the mean pointing error '''

        angles = []
        readItr = open(self.pointErrorFilePath, 'r')
        
        while(1):
            try:
                words = string.split(readItr.next().strip())
                if words[0].isdigit() == False:
                	continue
                angles.append(words[1])
            except StopIteration:
                print 'Excess path file parsed'
                break
        
        readItr.close()

        if len(angles) != self.delvPerEnv*self.totalEnv:
            print 'Some deliveries are missing'
            exit(1)
    
        # Write to a file
        writeItr = open(self.meanPointingError, 'w')
        thetas = []
        for d in range (0, self.delvPerEnv):
            
            for idx in range(0+d, self.delvPerEnv*self.totalEnv):
                if (idx-d)%15 == 0:
                    thetas.append(float(angles[idx]))

            if len(thetas) != self.totalEnv:
                print 'Invalid thetas size'
                exit(1)

            theta  = self.__calculateMeanDirection(thetas, len(thetas))
            writeItr.write(str(d) + '    ' + str(theta) + '\n')
            thetas = []
                
        writeItr.close() 
        
    def calculateBonus(self, maxBonus):
        ''' Used for calculating the bonus '''
        
        fileItr = open(self.meanPointingError, 'r')
        pointingError = 0.0
        numEnv = 0
        while(1):
            try:
                words = string.split(fileItr.next().strip())
                pointingError += math.degrees(float(words[1]))
                numEnv += 1
            except StopIteration:
                print 'Pointing-Error file done..'
                break
        
        fileItr.close()
        
        if pointingError == 0.0:
            print 'Pointing Error not initiated'
            exit(1)
            
        # Take Average
        pointingError /= numEnv
        
        scoreFiles = [self.logFilePath + self.subjectName + '_0_Score.log', self.logFilePath + self.subjectName + '_1_Score.log']
        
        if os.path.isfile(scoreFiles[0]) == False or os.path.isfile(scoreFiles[1]) == False:
            print 'Invalid Score file path'
            exit(0)
        
        numEnv = 0
        totalScore = 0.0
        
        for file in scoreFiles:
            fileItr = open(file, 'r')
            while(1):
                try:
                    words = string.split(fileItr.next().strip())
                    totalScore += float(words[2])
                    numEnv += 1
                except StopIteration:
                    print 'Score reading done..'
                    break
            fileItr.close()
        
        if totalScore == 0:
            print 'Incorrect Total Score'
            exit(1)
        
        totalScore /= numEnv
            
        pointingBonus = maxBonus*0.5*(180.0-pointingError)/180.0
        scoreBonus = min(maxBonus*0.5*totalScore/1050.0, 0.5*maxBonus)
#        print pointingBonus
#        print scoreBonus
        bonus = round(pointingBonus + scoreBonus)
        print 'Your bonus is: $%f' %(bonus)

#if __name__ == "__main__": 
#    createErrLogs = CreateErrorLogs('../data/', 'GB001')
#    createErrLogs.parseAndCreate()
#    createErrLogs.calculateMeanExcessPath()
#    createErrLogs.calculateMeanPointingError()
#    createErrLogs.calculateBonus(5.0)