#===============================================================================
# Calculate Bonus/Create Logs
# @author: Gaurav Bharaj
# @summary: * Calculate bonus based on two criteria:
#             1) Excess Path : Difference between actual Manhattan distance and 
#                             actual path
#             2) Pointing Error: Difference between actual direction and pointed 
#                               out direction (compass)
#           * Create a different log file for each delivery
#===============================================================================
import string
import re
import sys
import os.path

def checkCmdLineArgs(argv):
    ''' Check the command line arguments for correctness '''

    # Size should be three
    if len(argv) != 3:
        print 'Three arguments required, in the format:'
        print '>python CreateLogs.py [SubjectName] [SessionNum]'
        return False
    
    if argv[2] != '0' and argv[2] != '1':
        print 'Session Number should be either 0 or 1'
        return False
    
    if os.path.isfile(self.dataDirPath+argv[1]+'/session_'+argv[2]+'/log.txt') == False:
        print self.dataDirPath+argv[1]+'/session_'+argv[2]+'/log.txt',
        print ' does not exist'
        return False
    
    return True

class CreateLogs:
    ''' Parser the logs and creates new logs (with good information) '''
    
    def __init__(self, dataDirPath, subjectName, sessionNum):
        ''' Initiate the file path variables & Calculates the various file paths '''
        
        self.dataDirPath = dataDirPath
        if os.path.isdir(self.dataDirPath) == False:
            print 'Invalid data directory path'
            exit(0)
        
        self.subjectName = subjectName
        self.sessionNum = sessionNum
        self.inputLogFile = self.dataDirPath + self.subjectName + '/session_' + sessionNum + '/log.txt'
        
        self.outputLogDir = self.dataDirPath + self.subjectName + '/parsed/'
        if not os.path.exists(self.outputLogDir):
            os.makedirs(self.outputLogDir)
    
        self.scoreLogFile = self.outputLogDir + self.subjectName + '_'+ self.sessionNum +'_Score.log'
        self.createdLogFileName = self.outputLogDir + self.subjectName + '_LogNames.log'
        
        self.currentDelNumb = -1
        
        self.logFileName = None
        
        self.lineNumber = 0
    
    def createMoniterAvatarLogs(self):
        ''' 
        Separate from rest of logging, logs at the camera position & heading
        when the avatar is exploring, without a set goal 
        '''
        self.inputFileItr = open(self.inputLogFile, "r")
        self.currentDelNumb = -1  
        while(1):
            try:
                words = string.split(self.inputFileItr.next().strip())
                    
                if words[2] == 'SESSION_START':
                    sessionNum =  words[3]
                    while(1):
                        try:
                            words = string.split(self.inputFileItr.next().strip())
                            
                            if words[2] == 'ENVIRONMENT_START':
                                envNum =  words[4]
                                while(1):
                                    try:                      
                                        words = string.split(self.inputFileItr.next().strip())
                                                                                                                                                                   
                                        if words[2] == 'FORAGE_TASK_BEGUN':
                                            self.currentDelNumb += 1
                                                
                                            # Create Log file using SjN_SN_EN_DN.log 
                                            logFileName =   self.outputLogDir + self.subjectName + '_FORAGE_' + sessionNum + '_' + envNum + '_' + str(self.currentDelNumb%15+1) + '.log'                                
                                            logFile = open(logFileName,'w')
                                                
                                            while(1):
                                                try:
                                                    words = string.split(self.inputFileItr.next().strip())
                                                    self.lineNumber += 1
                                                        
                                                    # ColName        Time                Loc_avatar_x        Loc_avatar_y      Camera_Heading    
                                                    if words[2] == 'CAMERA_POS_HEAD':
                                                        loc_x = words[3][7:-1]
                                                        loc_y = words[4][0:-1]
                                                        lineToWrite = 'MOVE    ' + words[0] + '    ' + loc_x + '    ' + loc_y + '    ' + words[6] + '\n'
                                                        logFile.write(lineToWrite)
                                                       
                                                    # Check for DELIVERY_STORE_ARRIVE for end of delivery detection
                                                    if words [2] == 'FORAGE_TASK_ENDED':
                                                        logFile.close()
                                                        break
                                                except StopIteration:
                                                    print 'MONITER_TASK_ENDED excepted, but not found'
                                                    break
                                                    
                                        # Check for ENVIRONMENT_END for the environment & ignore the rest
                                        if words[2] == 'ENVIRONMENT_END':
                                            break
                                    except StopIteration:
                                        print 'ENVIRONMENT_END excepted, but not found'
                                        break
                            if words[2] == 'SESSION_END':
                                break
                        except StopIteration:
                            print 'SESSION_END excepted, but not found'
                            break         
            except StopIteration:
                print 'End of File Reached'
                break
            
        self.inputFileItr.close()        
            
    def getCompassData(self):
        ''' Parses the file to get the compass headings '''
        
        while(1):
            try:
                words = string.split(self.inputFileItr.next().strip())
                self.lineNumber += 1
                
                if words[2] == 'COMPASS_TASK_BEGUN':
                    initialCompassH = float(words[3])
                    cLoc_x = float(words[4][7:-1])
                    cLoc_y = float(words[5][0:-1])
                    print 'Compass Task'
    
                    while(1):
                        try:
                            words = string.split(self.inputFileItr.next().strip())
                            self.lineNumber += 1  
                            
                            if words[2] == 'AVATAR_OPTIMAL_HEADING':
                                optimalHeading = words[3]                     
                            
                            # Do all the compass stuff here                                         
                            if words[2] == 'COMPASS_TASK_ENDED':
                                finalCompassH = float(words[3])
                                return True, initialCompassH, finalCompassH, optimalHeading, cLoc_x, cLoc_y
                        
                        except StopIteration:
                            print 'COMPASS_TASK_ENDED expected, but not found'
                            return False, 0.0, 0.0, 0.0, 0.0, 0.0
                                                    
            except StopIteration:
                print 'Unexpected end of file'
                return False, 0.0, 0.0, 0.0, 0,0, 0.0
        
        return False, 0.0, 0.0, 0.0, 0.0, 0.0
                                                    
    def parseAndGenerate(self):
        ''' Parses through the log file and generates the logs '''
        
        slFileItr = open(self.scoreLogFile,'w')
        clFileItr = open(self.createdLogFileName,'a')
            
        self.inputFileItr = open(self.inputLogFile, "r")
        self.currentDelNumb = -1   
        while(1):
            try:
                words = string.split(self.inputFileItr.next().strip())
                self.lineNumber += 1
                    
                if words[2] == 'SESSION_START':
                    sessionNum =  words[3]
                    while(1):
                        try:
                            words = string.split(self.inputFileItr.next().strip())
                            self.lineNumber += 1
                    
                            if words[2] == 'FINAL_SCORE':
                                print 'Final Score Found'
                                lineToWrite = words[0] + '    ' + words[3] + '    ' + words[4] +'\n'
                                slFileItr.write(lineToWrite)  
    
                            initialCompassH = None
                            finalCompassH = None
    
                            if words[2] == 'ENVIRONMENT_START':
                                envNum =  words[4]
                                while(1):
                                    try:                      
                                        words = string.split(self.inputFileItr.next().strip())
                                        self.lineNumber += 1
                                                                                                                                                                   
                                        fileCreated = False
                                        if words[2] == 'DELIVERY_GOAL_DEFINED':
                                            self.currentDelNumb += 1
                                            deliveryName = words[3]
                                                
                                            # Create Log file using SjN_SN_EN_DN.log 
                                            if fileCreated == False:
                                                logFileName =   self.outputLogDir + self.subjectName + '_' + sessionNum + '_' + envNum + '_' + str(self.currentDelNumb%15+1) + '.log'                                
                                                logFile = open(logFileName,'w')
                                                clFileItr.write(self.subjectName + '_' + sessionNum + '_' + envNum + '_' + str(self.currentDelNumb%15+1) + '.log\n')
                                                    
                                                # Extract Goal(Location (x, y))
                                                loc_x = None
                                                loc_y = None
                                                
                                                for i, word in enumerate(words):
                                                    if re.match(r'Point3\S+', word) != None:
                                                        loc_x = word[7:-1]
                                                        loc_y = words[i+1][0:-1]
                                                        break
                                                           
                                                processed, initialCompassH, finalCompassH, optimalHeading, cLoc_x, cLoc_y = self.getCompassData()
                                                if  processed == True:
                                                    
                                                    # ColName        Name        Loc_goal_x        Loc_goal_y        Optimal_Heading(Theta)
                                                    lineToWrite = 'GOAL    ' + deliveryName + '    ' + loc_x + '    ' + loc_y + '    ' + optimalHeading + '\n' 
                                                    logFile.write(lineToWrite)
                                                        
                                                    # ColName        Loc_avatar_x        Loc_avatar_y        CompassHeading_Player(Theta)
                                                    lineToWrite = 'COMPASS    ' + str(initialCompassH) + '    ' + str(finalCompassH) + '    '+ str(cLoc_x) + '    ' + str(cLoc_y) +'\n'
                                                    logFile.write(lineToWrite)
                                                else:
                                                    print 'Compass task not processed'
                                                    
                                                fileCreated = True
                                                
                                            while(1):
                                                try:
                                                    words = string.split(self.inputFileItr.next().strip())
                                                    self.lineNumber += 1
                                                        
                                                    # ColName        Time                Loc_avatar_x        Loc_avatar_y      Camera_Heading    
                                                    if words[2] == 'CAMERA_POS_HEAD':
                                                        loc_x = words[3][7:-1]
                                                        loc_y = words[4][0:-1]
                                                        lineToWrite = 'MOVE    ' + words[0] + '    ' + loc_x + '    ' + loc_y + '    ' + words[6] + '\n'
                                                        logFile.write(lineToWrite)
                                                        
                                                    # SCORE       1273783557026         xxx
                                                    if words[2] == 'SCORE':
                                                        lineToWrite = 'SCORE    ' + words[0] + '    ' + words[3] +'\n'
                                                        logFile.write(lineToWrite)   
                                                       
                                                    # Check for DELIVERY_STORE_ARRIVE for end of delivery detection
                                                    if words [2] == 'DELIVERY_STORE_ARRIVE':
                                                        logFile.close()
                                                        break
                                                except StopIteration:
                                                    print 'DELIVERY_STORE_ARRIVE excepted, but not found'
                                                    break
                                                    
                                        # Check for ENVIRONMENT_END for the environment & ignore the rest
                                        if words[2] == 'ENVIRONMENT_END':
                                            break
                                    except StopIteration:
                                        print 'ENVIRONMENT_END excepted, but not found'
                                        break
                            if words[2] == 'SESSION_END':
                                break
                        except StopIteration:
                            print 'SESSION_END excepted, but not found'
                            break         
            except StopIteration:
                print 'End of File Reached'
                break
            
        self.inputFileItr.close()
        slFileItr.close()
        clFileItr.close()
        print 'Logging operation complete'   

#if __name__ == "__main__":
#    ''' Main run '''
    
#    if checkCmdLineArgs(sys.argv) == False:
#        exit(1)

#    genLogs = CreateLogs('../data/', sys.argv[1], sys.argv[2])
#    genLogs.parseAndGenerate()
#    genLogs.createMoniterAvatarLogs()