function Magellan_Data_Analysis_v2(datadir)
tic
% datadir-String-The locations of the subject parsed log folders
% 
% Creates 65 graphs per subject
%   
%   A learning curve for excess distance over destination number
%   A learning curve for pointing error over destination number
%   A bar graph for excess distance over difficulty level
%   A bar graph for pointing error over difficulty level
%   A scatterplot
%
%   And 60 graphs showing the path travelled for each destination
%
% Creates 4 Total graphs:
%
%   A learning curve for average excess distance over destination number
%   A learning curve for average pointing error over destination number
%   A bar graph for average excess distance over difficulty level
%   A bar graph for average pointing error over difficulty level
%
% 
%
% 7-7-10 TFL  Wrote it.
% 7-26-10 TFL V2


%% Set Up

% Folder Creation
figdir = fullfile(datadir,'Magellan_Data_Analysis');
mkdir(figdir);
cd(figdir);
% Constants
DELIVERIES_PER_ENVIRONMENT = 15;
SESSION_NUMBER = [0 1];
DIFFICULTY_LEVELS = 4;

% Current Completed Subjects and Condition Number
subjects = {'MAG001' 'MAG002' 'MAG003' 'MAG004' 'MAG005' 'MAG006' 'MAG007' 'MAG008' ...
            'MAG009' 'MAG010' 'MAG012' 'MAG013' 'MAG014' 'MAG015' 'MAG016' 'MAG017' ...
            'MAG018' 'MAG019' 'MAG020' 'MAG021' 'MAG022' 'MAG023' 'MAG024' 'MAG025'   ...
            'MAG026' 'MAG028' 'MAG029' 'MAG030' 'MAG031' 'MAG032' 'MAG033' 'MAG034' ...
            'MAG037' 'MAG040' 'MAG042' 'MAG044' 'MAG045' 'MAG046'};
cond = {1 2 3 4 5 6 7 8 ...
        1 2 3 4 5 6 4 1 ...
        2 4 3 8 8 7 3 5 ...
        1 7 6 8 6 2 5 7 ...
        6 4 3 5 8 2 };
num_subjects = length(subjects);

% Holder Array Generation
learning_curves_excess = zeros(num_subjects,DELIVERIES_PER_ENVIRONMENT);

%creating cross-subject scatterplot
learning_curves_excess_total = zeros(num_subjects*8,DELIVERIES_PER_ENVIRONMENT);


learning_curves_pe = zeros(num_subjects,DELIVERIES_PER_ENVIRONMENT);
learning_curves_pe2 = zeros(num_subjects,DELIVERIES_PER_ENVIRONMENT);

%creating cross-subject scatterplot
learning_curves_pe_total= zeros(num_subjects*8,DELIVERIES_PER_ENVIRONMENT);
learning_curves_pe2_total= zeros(num_subjects*8,DELIVERIES_PER_ENVIRONMENT);

bar_dat_excess = zeros(num_subjects,DIFFICULTY_LEVELS);
bar_dat_pe = zeros(num_subjects,DIFFICULTY_LEVELS);


%% Generate Data
count =0;
disp('Processing Subject Data')
for i = 1:num_subjects
    disp(subjects{i})
    
    % Generate Learning Curve Data
    lc_temp = zscore_learning_curves(datadir,subjects{i},SESSION_NUMBER);
    learning_curves_excess(i,:) = nanmean(lc_temp(1:8,:));
    
    %creating cross-subject scatterplot
    learning_curves_excess_total(((8*count)+1):(8*(count+1)) , : ) = lc_temp(1:8,:);
    
    learning_curves_pe_total(  ((8*count)+1):(8*(count+1)) , : ) = lc_temp(9:16,:).*(180/pi); % modified
    learning_curves_pe2_total(  ((8*count)+1):(8*(count+1)) , : )  = lc_temp(17:end,:).*(180/pi); % normal
    
    learning_curves_pe( i,: ) = circmean(lc_temp(9:16,:)).*(180/pi);
    learning_curves_pe2( i,: ) = circmean(lc_temp(17:end,:)).*(180/pi);

    bar_temp = bar_errors(datadir,subjects{i},SESSION_NUMBER,cond{i});
    bar_dat_excess(i,:) = bar_temp(1,:);
    bar_dat_pe(i,:) = bar_temp(2,:);
%     
     if exist(fullfile(datadir,subjects{i},'Paths'),'dir')
         disp(sprintf('Paths for %s have already been generated',subjects{i}))
     else
        % Generate Paths Graph
        all_path_display(datadir,[0 1],subjects{i})
    end
    close all
    
    count = count + 1;
end

disp('Finished Processing Subject Data')

%% Display Obtained Data

% disp('LC Excess')
% disp(learning_curves_excess)
% disp('LC PE')
% disp(learning_curves_pe)
% disp('Bar Excess')
% disp(bar_dat_excess)
% disp('Bar PE')
% disp(bar_dat_pe)
disp('Total Distance')
disp(learning_curves_excess_total)
disp('Total PE Modified')
disp(learning_curves_pe)
disp('Total PE Normal')
disp(learning_curves_pe2)


%% Average Data
excess_learning_curve = nanmean(learning_curves_excess);
excess_sems = nanstd(learning_curves_excess)/sqrt(size(learning_curves_excess,1));



 pe_learning_curve = circmean(learning_curves_pe);
 cmeans_sems = circsem(learning_curves_pe);
 
  pe_learning_curve2 = circmean(learning_curves_pe2);
 cmeans_sems2 = circsem(learning_curves_pe2);


dist_bar = nanmean(bar_dat_excess);
dist_bar_sem = nanstd(bar_dat_excess)/sqrt(size(bar_dat_excess,1));

pe_bar = circmean(bar_dat_pe);
cmeans_sems_bar = circsem(bar_dat_pe);


%% Graphing Data

fig_name = {'Excess_Learning_Curve' 'PE_Learning_Curve' 'Excess_Bar' 'PE_Bar' };


errorbar(1:DELIVERIES_PER_ENVIRONMENT,excess_learning_curve,excess_sems,excess_sems,'-k','LineWidth',2)
set(gca,'FontSize',18);
title 'Average Excess Distance'
xlabel('Delivery Number')
ylabel('Excess Distance')
xlim([0 16])

%print( gcf,'-dpdf', '-painters', fullfile(figdir,fig_name{1}));

figure
plot(learning_curves_excess_total,learning_curves_pe_total,'.k')

hold
reg = polyfit(learning_curves_excess_total,learning_curves_pe_total,1);
plot((-100:1500),(reg(1).*(-100:1500))+ reg(2),'r-')

title('Modified Summary Scatterplot')

figure
plot(learning_curves_excess_total,learning_curves_pe2_total,'.k')
hold
reg = polyfit(learning_curves_excess_total,learning_curves_pe2_total,1);
plot((-100:1500),(reg(1).*(-100:1500))+ reg(2),'r-')
title('Original Summary Scatterplot')

figure
plot(zscore(learning_curves_excess),circ_zscore(learning_curves_pe')','.k')
title('Modified Cross Subject/Collapsed Environment Scatterplot')
hold
reg = polyfit(zscore(learning_curves_excess),circ_zscore(learning_curves_pe')',1);
plot((-3:4),(reg(1).*(-3:4))+ reg(2),'r-')

figure
plot(zscore(learning_curves_excess),circ_zscore(learning_curves_pe2')','.k')
title('Original Cross Subject/Collapsed Environment Scatterplot')
hold
reg = polyfit(zscore(learning_curves_excess),circ_zscore(learning_curves_pe2')',1);
plot((-3:4),(reg(1).*(-3:4))+ reg(2),'r-')

figure

errorbar(1:DELIVERIES_PER_ENVIRONMENT,pe_learning_curve,cmeans_sems,cmeans_sems,'-k','LineWidth',2);

set(gca,'FontSize',18);
title 'Total Pointing Error Modified'
xlabel('Delivery Number')
ylabel('Pointing Error (deg)')
xlim([0 16])
%print( gcf,'-dpdf', '-painters', fullfile(figdir,fig_name2));

figure
errorbar(1:DELIVERIES_PER_ENVIRONMENT,pe_learning_curve2,cmeans_sems2,cmeans_sems2,'-k','LineWidth',2);

set(gca,'FontSize',18);
title 'Total Pointing Error Original'
xlabel('Delivery Number')
ylabel('Pointing Error (deg)')
xlim([0 16])
%print( gcf,'-dpdf', '-painters', fullfile(figdir,fig_name2));


figure
bar(1:DIFFICULTY_LEVELS,dist_bar)

hold;
errorbar(1:DIFFICULTY_LEVELS,dist_bar,dist_bar_sem,dist_bar_sem,'k.','LineWidth',2)
set(gca,'FontSize',18);
title 'Average Excess Distance over Difficulty Levels'
xlabel('Difficulty Level')
ylabel('Average Excess Distance')

 %   print( gcf,'-dpdf', '-painters', fullfile(figdir,fig_name{3}));


figure
bar(1:DIFFICULTY_LEVELS,pe_bar)

hold;
errorbar(1:DIFFICULTY_LEVELS,pe_bar,cmeans_sems_bar,cmeans_sems_bar,'k.','LineWidth',2)
set(gca,'FontSize',18);
title 'Average Pointing Error over Difficulty Levels'
xlabel('Delivery Number')
ylabel('Average Pointing Error (deg)')

%print( gcf,'-dpdf', '-painters', fullfile(figdir,fig_name{4}));

    toc


%% Calculates mean direction
function[cmean] = circmean(theta) %from Fisher p. 31
theta = theta.*(pi/180);
C = nansum(cos(theta),1);
S = nansum(sin(theta),1);
cmean = zeros(1,size(theta,2));

inds = (S > 0) & (C > 0); %case 1
cmean(inds) = atan(S(inds)./C(inds));

inds = C < 0; %case 2
cmean(inds) = atan(S(inds)./C(inds)) + pi;

inds = (S < 0) & (C > 0); %case 3
cmean(inds) = atan(S(inds)./C(inds)) + 2*pi;
cmean = cmean.*(180/pi);

%% Calculates Circular Stand Standard Deviation
function[csem] = circsem(theta) %from Fisher p. 32
theta = theta *(pi/180);
C = nansum(cos(theta),1);
S = nansum(sin(theta),1);
R = sqrt(C.^2 + S.^2);
n = size(theta,1);
R_bar = R./n;
V = 1 - R_bar;
v_minor = (-2*log(1-V)).^(1/2);
csem = v_minor/sqrt(n);
csem = csem *(180/pi);



