function total_data = bar_errors(datadir,subject,ses_nums,cond_num) 

% total_data is in virtual units and degrees!!!

% Cond1-ABCD-FGHE
% Cond2-BCDA-GHEF
% Cond3-CDAB-HEFG
% Cond4-DABC-EFGH
% Cond5-EFGH-BCDA
% Cond6-FGHE-CDAB
% Cond7-GHEF-DABC
% Cond8-HEFG-ABCD

env_orders = {'ABCDFGHE' 'BCDAGHEF' 'CDABHEFG' 'DABCEFGH' 'EFGHBCDA' 'FGHECDAB' 'GHEFDABC' 'HEFGABCD'};
cond = env_orders{cond_num};
env_difficulties = [1 2 3 4 1 2 3 4];
env_names = 'ABCDEFGH';

DELIVERIES_PER_ENVIRONMENT = 15;
ENVIRONMENTS_PER_SESSION = 4;

nSessions = length(ses_nums);

figdir = fullfile(datadir,subject,'Data');
mkdir(figdir);

dist_dat = zeros(length(unique(env_difficulties)),1);
pe_dat = nan(length(unique(env_difficulties)),DELIVERIES_PER_ENVIRONMENT*2);
pe_avg_dat = zeros(length(unique(env_difficulties)),1);

sesEnvCount = 1;
for sesNum = ses_nums
    
    for envNum = 0:ENVIRONMENTS_PER_SESSION-1
        for delivNum = 1:DELIVERIES_PER_ENVIRONMENT
            % Generates file name    
            
            fname = fullfile(datadir,subject,sprintf('%s_%d_%d_%d.log',subject,sesNum,envNum,delivNum)); 
            % Calculates excess_path and pointing_error, if the file exists
           
           
            if exist(fname,'file')      
                
                dist_dat(env_difficulties(strfind(env_names,cond(sesEnvCount))),1) = dist_dat(env_difficulties(strfind(env_names,cond(sesEnvCount))),1) + excess_path(fname);
                if isnan(pe_dat(env_difficulties(strfind(env_names,cond(sesEnvCount))),delivNum))
                    pe_dat(env_difficulties(strfind(env_names,cond(sesEnvCount))),delivNum) = pointing_error_v2(fname);
                else
                    pe_dat(env_difficulties(strfind(env_names,cond(sesEnvCount))),delivNum + 15) = pointing_error_v2(fname);
                end
                
                
                
                
            else
                warning('YCMAGELLAN:MissingFile','missing file: %s',fname);
            end
          
        end
        sesEnvCount = sesEnvCount + 1;
    end
end



dist_dat = dist_dat ./ (DELIVERIES_PER_ENVIRONMENT*2);

pe_avg_dat = circmean(pe_dat').*(180/pi);



figure;

bar(1:4,dist_dat)
title 'Average Excess Distance'
ylabel('Excess Distance')
xlabel('Difficulty')

    print( gcf,'-dpdf', '-painters', fullfile(datadir,subject,'Data',sprintf('%s_Bar_Excess',subject)));


figure

bar(1:4,pe_avg_dat)
title 'Average Pointing Error'
ylabel('Pointing Error (deg)')
xlabel('Difficulty')

    print( gcf,'-dpdf', '-painters', fullfile(datadir,subject,'Data',sprintf('%s_Bar_PE_v2',subject)));


total_data = [dist_dat' ;pe_avg_dat];


function dif = pointing_error(fname)
data = importdata(fname);
choice = normalize_angle(data.data(2,1));
dest = normalize_angle(data.data(1,3));
dif = normalize_angle(abs(choice-dest));

if dif > 180
    dif = 360 - dif;
end

dif = deg2rad(dif);

% Normalizes a given angle such that 0<=theta<360
function[theta] = normalize_angle(theta)
while theta < 0
    theta = theta + 360;
end
while theta > 360
    theta = theta - 360;
end

% Calculates mean direction
function[cmean] = circmean(theta) %from Fisher p. 31
C = nansum(cos(theta),1);
S = nansum(sin(theta),1);
cmean = zeros(1,size(theta,2));

inds = (S > 0) & (C > 0); %case 1
cmean(inds) = atan(S(inds)./C(inds));

inds = C < 0; %case 2
cmean(inds) = atan(S(inds)./C(inds)) + pi;

inds = (S < 0) & (C > 0); %case 3
cmean(inds) = atan(S(inds)./C(inds)) + 2*pi;


% Calculate Excess Path
function[exc] = excess_path(fname)
data=importdata(fname);
moves = strcmp(data.textdata(:,1),'MOVE');
goal_x = data.data(1,1);
goal_y = data.data(1,2);
x = data.data(moves,1);
y = data.data(moves,2);

% Calculate Ideal (City Block) Path Length
ideal = abs(x(1) - goal_x) + abs(y(1) - goal_y);

% Calculate Actual (Euclidean) Path Length
dists = sqrt((x(1:end-1) - x(2:end)).^2 + (y(1:end-1) - y(2:end)).^2);

exc = sum(dists) - ideal;


%Calculates the standard error of mean for excess path length
% mean_excess = nanmean(envir_dat(:,1));
% 
% std_excess = nanstd(envir_dat(:,1));
% 
% envir_dat(:,1)= (envir_dat(:,1)-mean_excess)/std_excess;
% 
% % Calculates the circular standard error of mean for pointing error
% pe_means = nanmean(envir_dat(:,2));
% std_pointing = circsem(envir_dat(:,2));
% 
% envir_dat(:,2)= (envir_dat(:,2)-pe_means)/std_pointing;



%bar_dat = [z_excess' z_thetas'] ;