function total_data = learning_curves(datadir,subject,ses_nums) % [excess,excess_sems,cmeans,cmeans_sems]
%LEARNING_CURVES     returns mean learning curve (excess path + pointing
%                    errors) for one subject's ycMagellan data
%     datadir-Location of subject data folder 
%       ex: '/Users/lewti/Documents/MATLAB/Magellan Data Analysis'
%     subject-Subject Name      
%       ex: 'TL001'
%     ses_nums-Number of sessions the subject has gone through
%       ex: 2
%
%function[excess,excess_sems,cmeans,cmeans_sems] = compass_learning_curve(dataroot,subj,ses_nums)
%
%6-15-10 TL   Wrote it.
%6-16-10 JRM  optimized, re-organized, folded pointing_Error.m (by TL) into
%             this function.  also folded in all_Dist_Length (by TL) and
%             its associated sub-functions.

DELIVERIES_PER_ENVIRONMENT = 15;
ENVIRONMENTS_PER_SESSION = 4;
nSessions = length(ses_nums);

excess = nan(nSessions*ENVIRONMENTS_PER_SESSION,DELIVERIES_PER_ENVIRONMENT);
thetas = nan(size(excess));
thetas2 = nan(size(excess));

figdir = fullfile(datadir,subject,'Data');
mkdir(figdir);

sesEnvCount = 1;
for sesNum = ses_nums   
    for envNum = 0:ENVIRONMENTS_PER_SESSION-1
        for delivNum = 1:DELIVERIES_PER_ENVIRONMENT
            % Generates file name    
            
            fname = fullfile(datadir,subject,sprintf('%s_%d_%d_%d.log',subject,sesNum,envNum,delivNum)); %,deliveryCount));
            % Calculates excess_path and pointing_error, if the file exists
            %disp(fname)
            if exist(fname,'file')
                excess(sesEnvCount,delivNum) = excess_path(fname);
                thetas(sesEnvCount,delivNum) = pointing_error_v2(fname); 
                thetas2(sesEnvCount,delivNum) = pointing_error(fname);
            else
                warning('YCMAGELLAN:MissingFile','missing file: %s',fname);
            end
        end
        sesEnvCount = sesEnvCount + 1;
    end
end



figure;

% Calculates the standard error of mean for excess path length
mean_excess = nanmean(excess);
excess_sems = nanstd(excess)/sqrt(size(excess,1));

% Calculates the circular standard error of mean for pointing error-store
% range
cmeans = circmean(thetas).*(180/pi);
cmeans_sems = circsem(thetas).*(180/pi);

% Calculates the circular standard error of mean for pointing error-center
% range
cmeans2 = circmean(thetas2).*(180/pi);
cmeans_sems2 = circsem(thetas2).*(180/pi);

total_data = [mean_excess ; cmeans;cmeans2];

% Graphs Excess Path Length
errorbar(1:DELIVERIES_PER_ENVIRONMENT,mean_excess,excess_sems,excess_sems,'k','LineWidth',2);
set(gca,'FontSize',18);
title 'Excess Path Length'
xlabel('Delivery number');
ylabel('Excess path (AU)');
xlim([0 DELIVERIES_PER_ENVIRONMENT+1]);


    print( gcf,'-dpdf', '-painters', fullfile(datadir,subject,'Data',sprintf('%s_Excess_LC',subject)));


figure;


% Graphs Pointing Error
errorbar(1:DELIVERIES_PER_ENVIRONMENT,cmeans,cmeans_sems,cmeans_sems,'k','LineWidth',2);
set(gca,'FontSize',18);
title 'Pointing Error'
xlabel('Delivery number');
ylabel('Pointing error (deg)');
xlim([0 DELIVERIES_PER_ENVIRONMENT+1]);

    print( gcf,'-dpdf', '-painters', fullfile(datadir,subject,'Data',sprintf('%s_PE_LC_v2',subject)));


figure;

%Graphs Excess Path Length vs. Pointing Error
plot(mean_excess,cmeans,'o','MarkerSize',5,'MarkerFaceColor','k','MarkerEdgeColor','k');
set(gca,'FontSize',18);
title 'Path Length vs. Pointing Error'
xlabel('Excess path');
ylabel('Pointing error (deg)');


    print( gcf,'-dpdf', '-painters', fullfile(datadir,subject,'Data',sprintf('%s_Correlation_v2',subject)));



% Calculates mean direction
function[cmean] = circmean(theta) %from Fisher p. 31
C = nansum(cos(theta),1);
S = nansum(sin(theta),1);
cmean = zeros(1,size(theta,2));

inds = (S > 0) & (C > 0); %case 1
cmean(inds) = atan(S(inds)./C(inds));

inds = C < 0; %case 2
cmean(inds) = atan(S(inds)./C(inds)) + pi;

inds = (S < 0) & (C > 0); %case 3
cmean(inds) = atan(S(inds)./C(inds)) + 2*pi;


% Calculates Circular Stand Standard Deviation
function[csem] = circsem(theta) %from Fisher p. 32
C = nansum(cos(theta),1);
S = nansum(sin(theta),1);
R = sqrt(C.^2 + S.^2);
n = size(theta,1);
R_bar = R./n;
V = 1 - R_bar;
v_minor = (-2*log(1-V)).^(1/2);
csem = v_minor/sqrt(n);


% Calculate the difference between the chosen angle and the correct angle
function dif = pointing_error(fname)
data = importdata(fname);
choice = normalize_angle(data.data(2,1));
dest = normalize_angle(data.data(1,3));
dif = normalize_angle(abs(choice-dest));

if dif > 180
    dif = 360 - dif;
end

dif = deg2rad(dif);

% Non-normalized Pointing Error-For PE scatterplot
% function dif = pointing_error2(fname)
% data = importdata(fname);
% choice = normalize_angle(data.data(2,1));
% dest = normalize_angle2(data.data(1,3));
% dif = normalize_angle2(abs(choice-dest));
% 
% dif = deg2rad(dif);

% Normalizes a given angle such that 0<=theta<360
function[theta] = normalize_angle(theta)
while theta < 0
    theta = theta + 360;
end
while theta > 360
    theta = theta - 360;
end

function[theta] = normalize_angle2(theta)
while theta < -360
    theta = theta + 360;
end
while theta > 360
    theta = theta - 360;
end


% Calculate Excess Path
function[exc] = excess_path(fname)
data=importdata(fname);
moves = strcmp(data.textdata(:,1),'MOVE');
goal_x = data.data(1,1);
goal_y = data.data(1,2);
x = data.data(moves,1);
y = data.data(moves,2);

% Calculate Ideal (City Block) Path Length
ideal = abs(x(1) - goal_x) + abs(y(1) - goal_y);

% Calculate Actual (Euclidean) Path Length
dists = sqrt((x(1:end-1) - x(2:end)).^2 + (y(1:end-1) - y(2:end)).^2);

exc = sum(dists) - ideal;