#===============================================================================
# Yellow Cab Experiment
# @author: Gaurav Bharaj
# @summary: This is the Experiment startup module
#===============================================================================
from pandaepl.common import *
from pandaepl.Options import Options
from pandaepl.Experiment import Experiment
from pandaepl.EnvironmentConf import EnvironmentConf
from pandaepl.Conf import Conf
#from pandaepl.Instructions import Instructions
import codecs
import ycCode

# Command-line run command
#python yellowCab.py -s test --no-eeg --show-fps --no-fs --resolution=1024x768 --envGen=P-ABCD-FGHE
#python yellowCab.py -s test --no-eeg --no-fs --resolution=1024x768 --envGen=P-ABCD-FGHE
   
def prepare(exp, config, envConfig):
    ''' Initial preparation for the experiment '''
    
    state = exp.getState()
        
    # set state for first session
    state =  { 'sessionNum'   : 0                     # Current session
             , 'sessionEnvNum': 0                     # Current environment 
             , 'sessionDone'  : config['sessionDone'] # There are two sessions
             , 'doPractice'   : config['doPractice']  # Check if  practice is complete
             , 'envDone'      : config['envDone']     # Check if an environment is done with 
             , 'doSFP'        : config['doSFP']       # Check if Store front practice is required
             } 
    
    # reset session to zero and save state
    exp.setSessionNum(0)
    exp.setState(state)

def run(exp,config, envConfig):
    ''' Main run function '''
    
    state = exp.getState()

    # instantiate yellow cab class
    city = ycCode.yellowCab(exp,config, envConfig, state) 

    # handle instructions
    if state['sessionNum'] == 0 and state['sessionEnvNum'] == 0 and state['doPractice'] == True:
        Instructions('PRACTICE_BEGIN_INSTRUCT', codecs.open(config['practInstructFile'],'r',encoding=config['encoding']).read()).display()
        
    if state['sessionNum'] > 0 and state['sessionEnvNum'] == 0:
        Instructions('WELCOME_BACK_INSTRUCT', codecs.open(config['welcomeBackInstruct'],'r',encoding=config['encoding']).read()).display()
        
    # start session
    city.run()

if __name__ == "__main__":
    
    # Check for correct env
    options = Options.getInstance().get()
    if options.envGen==None:
        Options.getInstance().error("Please specify environment generation string.")
         
    # get experiment instance
    exp = Experiment.getInstance()
    
    if not exp.getState():
        # must initialize session to get access to config
        exp.setSessionNum(0)
        config = Conf.getInstance().getConfig()
        envConfig = EnvironmentConf.getInstance().getConfig()
        prepare(exp, config, envConfig)
        run(exp,config, envConfig)
        exp.start()
    else:
        state = exp.getState()
        exp.setSessionNum(state['sessionNum'])
        config = Conf.getInstance().getConfig()
        envConfig = EnvironmentConf.getInstance().getConfig()
        run(exp,config, envConfig)
        exp.start()